@@ -1,6 +1,5 @@
 """Action to support type hints."""
 
-import copy
 import inspect
 import os
 import re
@@ -32,9 +31,9 @@ from typing import (
     Union,
 )
 
-from .actions import _find_action, _find_parent_action, _is_action_value_list
+from .actions import _ActionHelpClassPath, _find_action, _find_parent_action, _is_action_value_list
 from .loaders_dumpers import get_loader_exceptions, load_value
-from .namespace import is_empty_namespace, Namespace
+from .namespace import Namespace, split_key_root
 from .typing import is_final_class, registered_types
 from .optionals import (
     argcomplete_warn_redraw_prompt,
@@ -43,17 +42,20 @@ from .optionals import (
 )
 from .util import (
     change_to_path_dir,
+    ClassType,
     get_import_path,
     import_object,
+    indent_text,
+    is_subclass,
+    iter_to_set_str,
+    lenient_check,
+    lenient_check_context,
+    NoneType,
     object_path_serializer,
     ParserError,
+    parse_value_or_config,
     Path,
-    NoneType,
-    indent_text,
-    _issubclass,
-    _parse_value_or_config,
     warning,
-    lenient_check,
 )
 
 
@@ -131,6 +133,7 @@ class ActionTypeHint(Action):
             if 'metavar' not in kwargs:
                 kwargs['metavar'] = typehint_metavar(self._typehint)
             super().__init__(**kwargs)
+            self._supports_append = self.supports_append(self._typehint)
             self.normalize_default()
 
 
@@ -144,17 +147,32 @@ class ActionTypeHint(Action):
             self.default = get_import_path(default)
 
 
+    @staticmethod
+    def prepare_add_argument(args, kwargs, enable_path, container, sub_add_kwargs=None):
+        if 'action' in kwargs:
+            raise ValueError('Providing both type and action allowed.')
+        typehint = kwargs.pop('type')
+        if ActionTypeHint.supports_append(typehint):
+            args = tuple(list(args)+[args[0]+'+'])
+        if ActionTypeHint.is_subclass_typehint(typehint):
+            help_action = container.add_argument(args[0]+'.help', action=_ActionHelpClassPath(baseclass=typehint))
+            if sub_add_kwargs:
+                help_action.sub_add_kwargs = sub_add_kwargs
+        kwargs['action'] = ActionTypeHint(typehint=typehint, enable_path=enable_path)
+        return args
+
+
     @staticmethod
     def is_supported_typehint(typehint, full=False):
         """Whether the given type hint is supported."""
         supported = \
             typehint in root_types or \
-            getattr(typehint, '__origin__', None) in root_types or \
+            get_typehint_origin(typehint) in root_types or \
             typehint in registered_types or \
-            _issubclass(typehint, Enum) or \
+            is_subclass(typehint, Enum) or \
             ActionTypeHint.is_subclass_typehint(typehint)
         if full and supported:
-            typehint_origin = getattr(typehint, '__origin__', typehint)
+            typehint_origin = get_typehint_origin(typehint) or typehint
             if typehint not in root_types and typehint_origin in root_types and typehint_origin != Literal:
                 for typehint in getattr(typehint, '__args__', []):
                     if not (
@@ -172,7 +190,7 @@ class ActionTypeHint(Action):
         typehint = typehint_from_action(typehint)
         if typehint is None:
             return False
-        typehint_origin = getattr(typehint, '__origin__', None)
+        typehint_origin = get_typehint_origin(typehint)
         if typehint_origin == Union:
             subtypes = [a for a in typehint.__args__ if a != NoneType]
             test = all if all_subtypes else any
@@ -181,12 +199,12 @@ class ActionTypeHint(Action):
             typehint not in leaf_or_root_types and \
             typehint not in registered_types and \
             typehint_origin is None and \
-            not _issubclass(typehint, (Path, Enum))
+            not is_subclass(typehint, (Path, Enum))
 
 
     @staticmethod
     def is_mapping_typehint(typehint):
-        typehint_origin = getattr(typehint, '__origin__', typehint)
+        typehint_origin = get_typehint_origin(typehint) or typehint
         if typehint in mapping_origin_types or typehint_origin in mapping_origin_types or is_optional(typehint, tuple(mapping_origin_types)):
             return True
         return False
@@ -223,6 +241,15 @@ class ActionTypeHint(Action):
             subclass_arg_parser.set(subparser)
 
 
+    @staticmethod
+    def discard_init_args_on_class_path_change(parser, cfg_to, cfg_from):
+        for action in [a for a in parser._actions if ActionTypeHint.is_subclass_typehint(a)]:
+            val_to = cfg_to.get(action.dest)
+            val_from = cfg_from.get(action.dest)
+            if is_subclass_spec(val_to) and is_subclass_spec(val_from):
+                discard_init_args_on_class_path_change(action, val_to, val_from)
+
+
     @staticmethod
     @contextmanager
     def subclass_arg_context(parser):
@@ -246,6 +273,28 @@ class ActionTypeHint(Action):
             parser._apply_actions(cfg)
 
 
+    @staticmethod
+    def supports_append(action):
+        typehint = typehint_from_action(action)
+        typehint_origin = get_typehint_origin(typehint)
+        return typehint and (
+            typehint_origin in sequence_origin_types or
+            (
+                typehint_origin == Union and
+                any(get_typehint_origin(x) in sequence_origin_types for x in typehint.__args__)
+            )
+        )
+
+    @staticmethod
+    def apply_appends(parser, cfg):
+        for key in [k for k in cfg.keys() if k.endswith('+')]:
+            action = _find_action(parser, key[:-1])
+            if ActionTypeHint.supports_append(action):
+                val = action._check_type(cfg[key], append=True, cfg=cfg)
+                cfg[key[:-1]] = val
+                cfg.pop(key)
+
+
     def serialize(self, value, dump_kwargs=None):
         sub_add_kwargs = getattr(self, 'sub_add_kwargs', {})
         with dump_kwargs_context(dump_kwargs):
@@ -280,20 +329,26 @@ class ActionTypeHint(Action):
                             cfg_dest['class_path'] = default['class_path']
                         except (KeyError, TypeError):
                             pass
-                    if 'class_path' not in cfg_dest:
-                        raise ParserError(f'Found {opt_str} but not yet known to which class_path this corresponds.')
-                    if not sub_opt.startswith('init_args.'):
+                    if isinstance(cfg_dest, str):
+                        cfg_dest = Namespace(class_path=cfg_dest)
+                    if split_key_root(sub_opt)[0] not in {'init_args', 'dict_kwargs'}:
                         sub_opt = 'init_args.' + sub_opt
-                    if len(sub_opt.split('.', 2)) == 3:
+                    if sub_opt.startswith('init_args.') and '.' in sub_opt[len('init_args.'):]:
                         val = NestedArg(key=sub_opt[len('init_args.'):], val=val)
                         sub_opt = 'init_args'
-                    cfg_dest[sub_opt] = val
+                    elif sub_opt.startswith('dict_kwargs.') and isinstance(val, str):
+                        val = load_value(val)
+                    if isinstance(cfg_dest, list):
+                        cfg_dest[-1][sub_opt] = val
+                    else:
+                        cfg_dest[sub_opt] = val
                     val = cfg_dest
-            val = self._check_type(val, cfg=cfg)
+            append = opt_str == f'--{self.dest}+'
+            val = self._check_type(val, append=append, cfg=cfg)
         args[1].update(val, self.dest)
 
 
-    def _check_type(self, value, cfg=None):
+    def _check_type(self, value, append=False, cfg=None):
         islist = _is_action_value_list(self)
         if not islist:
             value = [value]
@@ -301,27 +356,36 @@ class ActionTypeHint(Action):
             try:
                 orig_val = val
                 try:
-                    val, config_path = _parse_value_or_config(val, enable_path=self._enable_path)
+                    val, config_path = parse_value_or_config(val, enable_path=self._enable_path)
                 except get_loader_exceptions():
                     config_path = None
                 path_meta = val.pop('__path__', None) if isinstance(val, dict) else None
-                sub_add_kwargs = getattr(self, 'sub_add_kwargs', {})
-                prev_val = cfg.get(self.dest) if cfg else None
+                kwargs = {
+                    'sub_add_kwargs': getattr(self, 'sub_add_kwargs', {}),
+                    'prev_val': cfg.get(self.dest) if cfg else None,
+                    'append': append,
+                }
                 try:
                     with change_to_path_dir(config_path):
-                        val = adapt_typehints(val, self._typehint, prev_val=prev_val, sub_add_kwargs=sub_add_kwargs)
+                        val = adapt_typehints(val, self._typehint, **kwargs)
                 except ValueError as ex:
                     val_is_int_float_or_none = isinstance(val, (int, float)) or val is None
                     if lenient_check.get():
                         value[num] = orig_val if val_is_int_float_or_none else val
                         continue
                     if val_is_int_float_or_none and config_path is None:
-                        val = adapt_typehints(orig_val, self._typehint, prev_val=prev_val, sub_add_kwargs=sub_add_kwargs)
+                        val = adapt_typehints(orig_val, self._typehint, **kwargs)
                     else:
                         if self._enable_path and config_path is None and isinstance(orig_val, str):
                             msg = f'\n- Expected a config path but "{orig_val}" either not accessible or invalid.\n- '
                             raise type(ex)(msg+str(ex)) from ex
                         raise ex
+
+                if not append and self._supports_append:
+                    prev_val = kwargs.get('prev_val')
+                    if isinstance(prev_val, list) and not_append_diff(prev_val, val) and get_typehint_origin(self._typehint) == Union:
+                        warnings.warn(f'Replacing list value "{prev_val}" with "{val}". To append to a list use "{self.dest}+".')
+
                 if path_meta is not None:
                     val['__path__'] = path_meta
                 if isinstance(val, (Namespace, dict)) and config_path is not None:
@@ -358,7 +422,7 @@ class ActionTypeHint(Action):
         if self.is_subclass_typehint(self, all_subtypes=False):
             class_paths = get_all_subclass_paths(self._typehint)
             if class_paths:
-                extra = ', known class paths: '+', '.join(class_paths)
+                extra = ', known subclasses: '+', '.join(class_paths)
         return extra
 
 
@@ -368,7 +432,7 @@ class ActionTypeHint(Action):
             return ['true', 'false']
         elif is_optional(self._typehint, bool):
             return ['true', 'false', 'null']
-        elif _issubclass(self._typehint, Enum):
+        elif is_subclass(self._typehint, Enum):
             enum = self._typehint
             return list(enum.__members__.keys())
         elif is_optional(self._typehint, Enum):
@@ -388,25 +452,26 @@ class ActionTypeHint(Action):
             return argcomplete_warn_redraw_prompt(prefix, msg)
 
 
-def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, prev_val=None, sub_add_kwargs=None):
+def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, prev_val=None, append=False, sub_add_kwargs=None):
 
     adapt_kwargs = {
         'serialize': serialize,
         'instantiate_classes': instantiate_classes,
         'prev_val': prev_val,
+        'append': append,
         'sub_add_kwargs': sub_add_kwargs or {},
     }
-    subtypehints = getattr(typehint, '__args__', None)
-    typehint_origin = getattr(typehint, '__origin__', typehint)
+    subtypehints = get_typehint_subtypes(typehint, append=append)
+    typehint_origin = get_typehint_origin(typehint) or typehint
 
     # Any
     if typehint == Any:
         type_val = type(val)
-        if type_val in registered_types or _issubclass(type_val, Enum):
+        if type_val in registered_types or is_subclass(type_val, Enum):
             val = adapt_typehints(val, type_val, **adapt_kwargs)
         elif isinstance(val, str):
             try:
-                val, _ = _parse_value_or_config(val, enable_path=False)
+                val, _ = parse_value_or_config(val, enable_path=False)
             except get_loader_exceptions():
                 pass
 
@@ -435,7 +500,7 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
             val = registered_type.deserializer(val)
 
     # Enum
-    elif _issubclass(typehint, Enum):
+    elif is_subclass(typehint, Enum):
         if serialize:
             if isinstance(val, typehint):
                 val = val.name
@@ -453,7 +518,7 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
             path = val
             val = import_object(val)
             if (typehint in {Type, type} and not isinstance(val, type)) or \
-               (typehint not in {Type, type} and not _issubclass(val, subtypehints[0])):
+               (typehint not in {Type, type} and not is_subclass(val, subtypehints[0])):
                 raise ValueError(f'Value "{path}" is not a {typehint}.')
 
     # Union
@@ -490,11 +555,22 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
 
     # List, Iterable or Sequence
     elif typehint_origin in sequence_origin_types:
+        if append:
+            if prev_val is None:
+                prev_val = []
+            elif not isinstance(prev_val, list):
+                try:
+                    prev_val = [adapt_typehints(prev_val, subtypehints[0], **adapt_kwargs)]
+                except Exception:
+                    prev_val = []
+            val = prev_val + (val if isinstance(val, list) else [val])
+            prev_val = prev_val + [None] * (len(val) if isinstance(val, list) else 1)
         if not isinstance(val, list):
             raise ValueError(f'Expected a List but got "{val}"')
         if subtypehints is not None:
             for n, v in enumerate(val):
-                val[n] = adapt_typehints(v, subtypehints[0], **adapt_kwargs)
+                adapt_kwargs_n = {**adapt_kwargs, 'prev_val': prev_val[n]} if isinstance(prev_val, list) else adapt_kwargs
+                val[n] = adapt_typehints(v, subtypehints[0], **adapt_kwargs_n)
 
     # Dict, Mapping
     elif typehint_origin in mapping_origin_types:
@@ -506,7 +582,7 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
                 val = {cast(k): v for k, v in val.items()}
             for k, v in val.items():
                 if "linked_targets" in adapt_kwargs["sub_add_kwargs"]:
-                    kwargs = copy.deepcopy(adapt_kwargs)
+                    kwargs = deepcopy(adapt_kwargs)
                     sub_add_kwargs = kwargs["sub_add_kwargs"]
                     sub_add_kwargs["linked_targets"] = {t[len(k + "."):] for t in sub_add_kwargs["linked_targets"]
                                                         if t.startswith(k + ".")}
@@ -519,10 +595,8 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
     # Callable
     elif typehint_origin in callable_origin_types or typehint in callable_origin_types:
         if serialize:
-            if is_class_object(val):
-                class_path = val['class_path']
-                init_args = val.get('init_args', Namespace())
-                val['init_args'] = adapt_class_type(class_path, init_args, True, False, sub_add_kwargs)
+            if is_subclass_spec(val):
+                val = adapt_class_type(val, True, False, sub_add_kwargs)
             else:
                 val = object_path_serializer(val)
         else:
@@ -536,19 +610,13 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
                     else:
                         raise ImportError(f'Unexpected import object {val_obj}')
                 if isinstance(val, (dict, Namespace)):
-                    if not is_class_object(val):
+                    if not is_subclass_spec(val):
                         raise ImportError(f'Dict must include a class_path and optionally init_args, but got {val}')
-                    val = Namespace(val)
-                    val_class = import_object(val.class_path)
+                    val_class = import_object(val['class_path'])
                     if not (inspect.isclass(val_class) and callable_instances(val_class)):
-                        raise ImportError(f'"{val.class_path}" is not a callable class.')
+                        raise ImportError(f'{val["class_path"]!r} is not a callable class.')
                     val['class_path'] = get_import_path(val_class)
-                    init_args = val.get('init_args', Namespace())
-                    adapted = adapt_class_type(val_class, init_args, False, instantiate_classes, sub_add_kwargs)
-                    if instantiate_classes and sub_add_kwargs.get('instantiate', True):
-                        val = adapted
-                    elif adapted is not None and not is_empty_namespace(adapted):
-                        val['init_args'] = adapted
+                    val = adapt_class_type(val, False, instantiate_classes, sub_add_kwargs)
             except (ImportError, AttributeError, ParserError) as ex:
                 raise ValueError(f'Type {typehint} expects a function or a callable class: {ex}')
 
@@ -560,31 +628,26 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
             return val
         if serialize and isinstance(val, str):
             return val
-        if not (isinstance(val, str) or is_class_object(val)):
-            raise ValueError(f'Type {typehint} expects an str or a Dict/Namespace with a class_path entry but got "{val}"')
+
+        val_input = val
+        val = subclass_spec_as_namespace(val)
+        if not is_subclass_spec(val) and prev_val and 'class_path' in prev_val:
+            if 'init_args' in val:
+                val['class_path'] = prev_val['class_path']
+            else:
+                val = Namespace(class_path=prev_val['class_path'], init_args=val)
+        if not is_subclass_spec(val):
+            raise ValueError(
+                f'Type {typehint} expects: a class path (str); or a dict with a class_path entry; '
+                f'or a dict with init_args (if class path given previously). Got "{val_input}".'
+            )
+
         try:
-            if isinstance(val, str):
-                val = Namespace(class_path=val)
-            elif isinstance(val, dict):
-                val = Namespace(val)
             val_class = import_object(resolve_class_path_by_name(typehint, val['class_path']))
-            if not _issubclass(val_class, typehint):
+            if not is_subclass(val_class, typehint):
                 raise ValueError(f'"{val["class_path"]}" is not a subclass of {typehint}')
-            val['class_path'] = class_path = get_import_path(val_class)
-            if isinstance(prev_val, Namespace) and 'class_path' in prev_val and 'init_args' not in val:
-                prev_class_path = prev_val['class_path']
-                prev_init_args = prev_val.get('init_args')
-                if prev_class_path != class_path and not (prev_init_args is None or is_empty_namespace(prev_init_args)):
-                    warnings.warn(
-                        f'Changing class_path to {class_path} implies discarding init_args {prev_init_args.as_dict()} '
-                        f'defined for class_path {prev_class_path}.'
-                    )
-            init_args = val.get('init_args', Namespace())
-            adapted = adapt_class_type(val_class, init_args, serialize, instantiate_classes, sub_add_kwargs, prev_val=prev_val)
-            if instantiate_classes and sub_add_kwargs.get('instantiate', True):
-                val = adapted
-            elif adapted is not None and not is_empty_namespace(adapted):
-                val['init_args'] = adapted
+            val['class_path'] = get_import_path(val_class)
+            val = adapt_class_type(val, serialize, instantiate_classes, sub_add_kwargs, prev_val=prev_val)
         except (ImportError, AttributeError, AssertionError, ParserError) as ex:
             class_path = val if isinstance(val, str) else val['class_path']
             e = indent_text(f'\n- {ex}')
@@ -593,14 +656,24 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
     return val
 
 
-def is_class_object(val):
+def is_subclass_spec(val):
     is_class = isinstance(val, (dict, Namespace)) and 'class_path' in val
     if is_class:
         keys = getattr(val, '__dict__', val).keys()
-        is_class = len(set(keys)-{'class_path', 'init_args', '__path__'}) == 0
+        is_class = len(set(keys)-{'class_path', 'init_args', 'dict_kwargs', '__path__'}) == 0
     return is_class
 
 
+def subclass_spec_as_namespace(val):
+    if isinstance(val, str):
+        return Namespace(class_path=val)
+    if isinstance(val, dict):
+        val = Namespace(val)
+    if 'init_args' in val and isinstance(val['init_args'], dict):
+        val['init_args'] = Namespace(val['init_args'])
+    return val
+
+
 class NestedArg:
     def __init__(self, key, val):
         self.key = key
@@ -625,7 +698,7 @@ def get_all_subclass_paths(cls: Type) -> List[str]:
         for subclass in cl.__subclasses__() if hasattr(cl, '__subclasses__') else []:
             add_subclasses(subclass)
 
-    if getattr(cls, '__origin__', None) == Union:
+    if get_typehint_origin(cls) == Union:
         for arg in cls.__args__:
             if ActionTypeHint.is_subclass_typehint(arg):
                 add_subclasses(arg)
@@ -662,9 +735,34 @@ def dump_kwargs_context(kwargs):
     yield
 
 
-def adapt_class_type(val_class, init_args, serialize, instantiate_classes, sub_add_kwargs, prev_val=None):
-    if isinstance(init_args, dict):
-        init_args = Namespace(init_args)
+def discard_init_args_on_class_path_change(parser_or_action, prev_val, value):
+    if prev_val and 'init_args' in prev_val and prev_val['class_path'] != value.class_path:
+        parser = parser_or_action
+        if isinstance(parser_or_action, ActionTypeHint):
+            sub_add_kwargs = getattr(parser_or_action, 'sub_add_kwargs', {})
+            parser = ActionTypeHint.get_class_parser(value.class_path, sub_add_kwargs)
+        prev_val = subclass_spec_as_namespace(prev_val)
+        del_args = {}
+        for key, val in list(prev_val.init_args.__dict__.items()):
+            action = _find_action(parser, key)
+            if action:
+                with lenient_check_context(lenient=False):
+                    try:
+                        parser._check_value_key(action, val, key, Namespace())
+                    except Exception:
+                        action = None
+            if not action:
+                del_args[key] = prev_val.init_args.pop(key)
+        if del_args:
+            warnings.warn(
+                f'Due to class_path change from {prev_val.class_path!r} to {value.class_path!r}, '
+                f'discarding init_args: {del_args}.'
+            )
+
+
+def adapt_class_type(value, serialize, instantiate_classes, sub_add_kwargs, prev_val=None):
+    value = subclass_spec_as_namespace(value)
+    val_class = import_object(value.class_path)
     parser = ActionTypeHint.get_class_parser(val_class, sub_add_kwargs)
 
     # No need to re-create the linked arg but just "inform" the corresponding parser actions that it exists upstream.
@@ -683,19 +781,68 @@ def adapt_class_type(val_class, init_args, serialize, instantiate_classes, sub_a
 
             break
 
+    discard_init_args_on_class_path_change(parser, prev_val, value)
+
+    dict_kwargs = value.pop('dict_kwargs', {})
+    init_args = value.get('init_args', Namespace())
+
     if instantiate_classes:
         init_args = parser.instantiate_classes(init_args)
         if not sub_add_kwargs.get('instantiate', True):
-            return init_args
-        return val_class(**init_args)
+            if init_args:
+                value['init_args'] = init_args
+            return value
+        return val_class(**{**init_args, **dict_kwargs})
+
+    if isinstance(init_args, NestedArg):
+        value['init_args'] = parser.parse_args(
+            [f'--{init_args.key}={init_args.val}'],
+            namespace=prev_val.init_args.clone(),
+            defaults=sub_defaults.get(),
+        )
+        return value
+
     if serialize:
-        init_args = None if is_empty_namespace(init_args) else load_value(parser.dump(init_args, **dump_kwargs.get()))
-    elif isinstance(init_args, NestedArg):
-        prev_val = prev_val.init_args.clone()
-        init_args = parser.parse_args([f'--{init_args.key}={init_args.val}'], namespace=prev_val, defaults=sub_defaults.get())
+        if init_args:
+            value['init_args'] = load_value(parser.dump(init_args, **dump_kwargs.get()))
     else:
+        if isinstance(dict_kwargs, str):
+            dict_kwargs = load_value(dict_kwargs)
+        if isinstance(dict_kwargs, (dict, Namespace)):
+            for key in list(dict_kwargs.keys()):
+                if _find_action(parser, key):
+                    init_args[key] = dict_kwargs.pop(key)
+            if isinstance(dict_kwargs, Namespace):
+                dict_kwargs = dict(dict_kwargs)
+        elif dict_kwargs:
+            init_args['dict_kwargs'] = dict_kwargs
+            dict_kwargs = None
         init_args = parser.parse_object(init_args, defaults=sub_defaults.get())
-    return init_args
+        if init_args:
+            value['init_args'] = init_args
+    if dict_kwargs:
+        value['dict_kwargs'] = dict_kwargs
+    return value
+
+
+def not_append_diff(val1, val2):
+    if isinstance(val1, list) and isinstance(val2, list):
+        val1 = [x.get('class_path') if is_subclass_spec(x) else x for x in val1]
+        val2 = [x.get('class_path') if is_subclass_spec(x) else x for x in val2]
+    return val1 != val2
+
+
+def get_typehint_subtypes(typehint, append):
+    subtypes = getattr(typehint, '__args__', None)
+    if append and subtypes:
+        subtypes = sorted(subtypes, key=lambda x: get_typehint_origin(x) not in sequence_origin_types)
+    return subtypes
+
+
+def get_typehint_origin(typehint):
+    if not hasattr(typehint, '__origin__') and get_import_path(typehint.__class__) == 'types.UnionType':
+        return Union
+    return getattr(typehint, '__origin__', None)
 
 
 def is_ellipsis_tuple(typehint):
@@ -704,23 +851,23 @@ def is_ellipsis_tuple(typehint):
 
 def is_optional(annotation, ref_type):
     """Checks whether a type annotation is an optional for one type class."""
-    return getattr(annotation, '__origin__', None) == Union and \
+    return get_typehint_origin(annotation) == Union and \
         len(annotation.__args__) == 2 and \
         any(NoneType == a for a in annotation.__args__) and \
-        any(_issubclass(a, ref_type) for a in annotation.__args__)
+        any(is_subclass(a, ref_type) for a in annotation.__args__)
 
 
 def is_enum_type(annotation):
-    return _issubclass(annotation, Enum) or \
-        (getattr(annotation, '__origin__', None) == Union and
-         any(_issubclass(a, Enum) for a in annotation.__args__))
+    return is_subclass(annotation, Enum) or \
+        (get_typehint_origin(annotation) == Union and
+         any(is_subclass(a, Enum) for a in annotation.__args__))
 
 
 def is_callable_type(annotation):
     def is_callable(a):
-        return getattr(a, '__origin__', a) in callable_origin_types or a in callable_origin_types
+        return (get_typehint_origin(a) or a) in callable_origin_types or a in callable_origin_types
     return is_callable(annotation) or \
-        (getattr(annotation, '__origin__', None) == Union and
+        (get_typehint_origin(annotation) == Union and
          any(is_callable(a) for a in annotation.__args__))
 
 
@@ -731,25 +878,35 @@ def typehint_from_action(action_or_typehint):
 
 
 def type_to_str(obj):
-    if _issubclass(obj, (bool, int, float, str, Enum)):
+    if obj in {bool, tuple} or is_subclass(obj, (int, float, str, Enum)):
         return obj.__name__
     elif obj is not None:
-        return re.sub(r'[a-z_.]+\.', '', str(obj)).replace('NoneType', 'null')
+        return re.sub(r'[A-Za-z0-9_<>.]+\.', '', str(obj)).replace('NoneType', 'null')
+
+
+def literal_to_str(val):
+    return 'null' if val is None else str(val)
 
 
 def typehint_metavar(typehint):
     """Generates a metavar for some types."""
     metavar = None
+    typehint_origin = get_typehint_origin(typehint) or typehint
     if typehint == bool:
         metavar = '{true,false}'
     elif is_optional(typehint, bool):
         metavar = '{true,false,null}'
-    elif _issubclass(typehint, Enum):
+    elif typehint_origin == Literal:
+        args = typehint.__args__
+        metavar = iter_to_set_str(literal_to_str(a) for a in args)
+    elif is_subclass(typehint, Enum):
         enum = typehint
-        metavar = '{'+','.join(list(enum.__members__.keys()))+'}'
+        metavar = iter_to_set_str(enum.__members__)
     elif is_optional(typehint, Enum):
         enum = typehint.__args__[0]
-        metavar = '{'+','.join(list(enum.__members__.keys())+['null'])+'}'
+        metavar = iter_to_set_str(list(enum.__members__.keys())+['null'])
+    elif typehint_origin in tuple_set_origin_types:
+        metavar = '[ITEM,...]'
     return metavar
 
 
@@ -821,9 +978,6 @@ class LazyInitBaseClass:
         return init
 
 
-ClassType = TypeVar('ClassType')
-
-
 def lazy_instance(class_type: Type[ClassType], **kwargs) -> ClassType:
     """Instantiates a lazy instance of the given type.
 
